using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using Telerik.Web.UI;

public partial class _Default : System.Web.UI.Page
{
        
    //Creates a datatable and adds it to the session to be used later .
    //The datatable is added to the session as it is being modified during the Drag And Drop operations.
    //Therefore, the "data" object always contains the proper rows and columns.
    private DataTable DataSource 
    {
        get
        {
            DataTable data = (DataTable)Session["DataSource"];
            if (data == null)
            {
                data = new DataTable();
                data.Columns.Add("Column1");
                data.Columns.Add("Column2");
                data.Rows.Add(new object[] { "item1", "value1" });
                data.Rows.Add(new object[] { "item2", "value2" });
                data.Rows.Add(new object[] { "item3", "value3" });
                Session["DataSource"]  = data;
            }
            
            return data;
        }
    }


    protected override void OnLoad(EventArgs e)
    {
        if (!Page.IsPostBack)
        {
            RadTreeView1.Nodes.Add(new RadTreeNode("root1", "rootValue1"));
            RadTreeView1.Nodes.Add(new RadTreeNode("root2", "rootValue2"));
        }

        base.OnLoad(e);
    }   
    

    protected void RadGrid1_ItemCreated(object sender, GridItemEventArgs e)
	{
        if (e.Item is GridDataItem)
        {
            GridDataItem dataItem = e.Item as GridDataItem;
            //Registers the onmousedown event to be fired upon selecting a grid's row and passes arguments to the event handler
            e.Item.Attributes["onmousedown"] = string.Format("onMouseDown(event, this,{0})", dataItem.DataSetIndex);
            e.Item.Style["cursor"] = "move";
        }
	}

	protected void RadAjaxManager1_AjaxRequest(object sender, AjaxRequestEventArgs e)
	{
        //This ajax request is fired upon dropping a grid's row onto a TreeNode
        
        //Finds the node, onto which the grid's row should be dropped, by text using the previously stored value of the "TreeNodeText" hidden field
        RadTreeNode destinationNode = RadTreeView1.FindNodeByText(TreeNodeText.Value);
        DataRow sourceRow = DataSource.Rows[Convert.ToInt32(DataSetIndex.Value)];

        //Creates a new node using the respective grid's row information. The text of the TreeNode is set to be the value of the needed cell in the grid's datasource
        RadTreeNode sourceNode = new RadTreeNode((string)sourceRow["Column1"], (string)sourceRow["Column2"]);

        destinationNode.Nodes.Add(sourceNode);
        destinationNode.Expanded = true;

        //Removes the row that has been dropped onto the treeview
        DataSource.Rows.Remove(sourceRow);

        //Rebinds the grid to address the changes made to its datasource
        RadGrid1.Rebind();
	}

    protected void RadGrid1_NeedDataSource(object source, GridNeedDataSourceEventArgs e)
    {       
        RadGrid1.DataSource = DataSource;
    }

    /// <summary>
    /// Fires when a Node is dropped onto the <see cref="RadTreeView"/> 'RadTreeView1'.
    /// </summary>
    /// <param name="sender">The 'RadTreeView1' object.</param>
    /// <param name="e">Event arguments.</param>
    protected void RadTreeView1_NodeDrop(object sender, RadTreeNodeDragDropEventArgs e)
    {
        if (e.DestDragNode != null)
        {
            //Handle when dropped on another Node.
        }
        else if (e.HtmlElementID == RadGrid1.ClientID)
        {
            DataTable dt = (DataTable)Session["DataSource"];

            foreach (RadTreeNode node in e.DraggedNodes)
            {
                AddRowToGrid(dt, node);

                AddRowsToGrid(dt, node.Nodes);

                node.Remove();
            }
        }        
    }

    /// <summary>
    /// Adds rows to the specified Data Table for
    /// every Node in the specified list of nodes and its child Nodes.
    /// </summary>
    /// <param name="dt">The <see cref="DataTable"/> object where rows are added.</param>
    /// <param name="nodes">A <see cref="RadTreeNodeCollection"/> for Nodes.</param>
    private void AddRowsToGrid(DataTable dt, RadTreeNodeCollection nodes)
    {
        foreach (RadTreeNode node in nodes)
        {
            AddRowToGrid(dt, node);

            AddRowsToGrid(dt, node.Nodes);
        }
    }

    /// <summary>
    /// Adds a new Row to the specified <see cref="DataTable"/> using data 
    /// from the specified <see cref="RadTreeNode"/> object and
    /// binds the <see cref="RadGrid"/> 'RadGrid1'.
    /// </summary>
    /// <param name="dt"></param>
    /// <param name="node"></param>
    private void AddRowToGrid(DataTable dt, RadTreeNode node)
    {
        string[] values = { node.Text, node.Value };
        dt.Rows.Add(values);

        RadGrid1.DataSource = dt;
        RadGrid1.DataBind();
    }
}